@@ -501,5 +501,9 @@
 #define CAN_USE_NLIST
 #endif
 
+#if HAVE_DMALLOC_H
+#define DMALLOC_FUNC_CHECK
+#endif
+
 #undef INET6
 
