@@ -41,6 +41,10 @@
 #endif
 #include <getopt.h>
 
+#if HAVE_DMALLOC_H
+#include <dmalloc.h>
+#endif
+
 #include "asn1.h"
 #include "snmp_api.h"
 #include "snmp_impl.h"
@@ -55,18 +59,10 @@
 #include "agent_callbacks.h"
 #include "tools.h"
 #include "snmp_logging.h"
+#include "callback.h"
 
 #include "mib_module_config.h"
 
-#ifdef USING_NOTIFICATION_SNMPNOTIFYTABLE_MODULE
-#include "snmp_vars.h"
-#include "snmp-tc.h"
-#include "target/snmpTargetAddrEntry.h"
-#include "target/snmpTargetParamsEntry.h"
-#include "notification/snmpNotifyTable.h"
-#endif
-
-
 struct trap_sink {
     struct snmp_session	*sesp;
     struct trap_sink	*next;
@@ -133,114 +129,32 @@ static void free_trap_session (struct trap_sink *sp)
 
 int add_trap_session( struct snmp_session *ss, int pdutype, int confirm, int version )
 {
-#ifdef USING_NOTIFICATION_SNMPNOTIFYTABLE_MODULE
-#define MAX_ENTRIES 1024
-    struct targetAddrTable_struct *ptr;
-    struct targetParamTable_struct *pptr;
-    struct snmpNotifyTable_data *nptr;
-    int i;
-    char buf[SNMP_MAXBUF_SMALL];
-    oid udpdomain[] = { 1,3,6,1,6,1,1 };
-    int udpdomainlen = sizeof(udpdomain)/sizeof(oid);
-#ifdef HAVE_GETHOSTBYNAME
-    struct hostent *hp;
-#endif
-
-    if (!ss)
-        return (0);
-
-        for(i=0; i < MAX_ENTRIES; i++) {
-            sprintf(buf, "internal%d", i);
-            if (get_addrForName(buf) == NULL && get_paramEntry(buf) == NULL)
-                break;
-        }
-        if (i == MAX_ENTRIES) {
-            snmp_sess_close(ss);
-            return(0);
-        }
-
-        /* address */
-        ptr = snmpTargetAddrTable_create();
-        ptr->name = strdup(buf);
-        memcpy(ptr->tDomain, udpdomain, udpdomainlen*sizeof(oid));
-        ptr->tDomainLen = udpdomainlen;
-
-#ifdef HAVE_GETHOSTBYNAME
-        hp = gethostbyname(ss->peername);
-        if (hp != NULL){
-            /* XXX: fix for other domain types */
-            ptr->tAddressLen = hp->h_length + 2;
-            ptr->tAddress = malloc(ptr->tAddressLen);
-            memmove(ptr->tAddress, hp->h_addr, hp->h_length);
-            ptr->tAddress[hp->h_length] = (ss->remote_port & 0xff00) >> 8;
-            ptr->tAddress[hp->h_length+1] = (ss->remote_port & 0xff);
-        } else {
-#endif /* HAVE_GETHOSTBYNAME */
-            ptr->tAddressLen = 6;
-            ptr->tAddress = calloc(1, ptr->tAddressLen);
-#ifdef HAVE_GETHOSTBYNAME
-        }
-#endif /* HAVE_GETHOSTBYNAME */
-        ptr->timeout = ss->timeout/1000;
-        ptr->retryCount = ss->retries;
-        ptr->tagList = strdup(ptr->name);
-        ptr->params = strdup(ptr->name);
-        ptr->storageType = ST_READONLY;
-        ptr->rowStatus = RS_ACTIVE;
-        ptr->sess = ss;
-        DEBUGMSGTL(("trapsess", "adding to trap table\n"));
-        snmpTargetAddrTable_add(ptr);
-
-        /* param */
-        pptr = snmpTargetParamTable_create();
-        pptr->paramName = strdup(buf);
-        pptr->mpModel = ss->version;
-        if (ss->version == SNMP_VERSION_3) {
-            pptr->secModel = ss->securityModel;
-            pptr->secLevel = ss->securityLevel;
-            memdup((void *) &pptr->secName, (void *) ss->securityName,
-                   ss->securityNameLen+1);
-	    pptr->secName[ss->securityNameLen] = 0;
-        } else {
-            pptr->secModel = ss->version;
-            pptr->secLevel = SNMP_SEC_LEVEL_NOAUTH;
-            pptr->secName = NULL;
-            if (ss->community && (ss->community_len > 0)) {
-                memdup((void *) &pptr->secName, (void *) ss->community,
-                   ss->community_len+1);
-		pptr->secName[ss->community_len] = 0;
-	    }
-        }
-        pptr->storageType = ST_READONLY;
-        pptr->rowStatus = RS_ACTIVE;
-        snmpTargetParamTable_add(pptr);
-            
-        /* notify table */
-        nptr = SNMP_MALLOC_STRUCT(snmpNotifyTable_data);
-        nptr->snmpNotifyName = strdup(buf);
-        nptr->snmpNotifyNameLen = strlen(buf);
-        nptr->snmpNotifyTag = strdup(buf);
-        nptr->snmpNotifyTagLen = strlen(buf);
-        nptr->snmpNotifyCommand = pdutype;
-        nptr->snmpNotifyType = confirm ?
-            SNMPNOTIFYTYPE_TRAP : SNMPNOTIFYTYPE_INFORM;
-        nptr->snmpNotifyStorageType = ST_READONLY;
-        nptr->snmpNotifyRowStatus = RS_ACTIVE;
-
-        snmpNotifyTable_add(nptr);
-#else /* ! USING_NOTIFICATION_SNMPNOTIFYTABLE_MODULE */
-    struct trap_sink *new_sink;
+    if (snmp_callback_available(SNMP_CALLBACK_APPLICATION,
+                                SNMPD_CALLBACK_REGISTER_NOTIFICATIONS) ==
+        SNMPERR_SUCCESS) {
+        /* something else wants to handle notification registrations */
+        struct agent_add_trap_args args;
+        DEBUGMSGTL(("add_trap_session","adding callback trap sink\n"));
+        args.ss = ss;
+        args.confirm = confirm;
+        snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
+                       SNMPD_CALLBACK_REGISTER_NOTIFICATIONS,
+                       (void *) &args);
+    } else {
+        /* no other support exists, handle it ourselves. */
+        struct trap_sink *new_sink;
     
-    new_sink = (struct trap_sink *) malloc (sizeof (*new_sink));
-    if ( new_sink == NULL )
-	return 0;
+        DEBUGMSGTL(("add_trap_session","adding internal trap sink\n"));
+        new_sink = (struct trap_sink *) malloc (sizeof (*new_sink));
+        if ( new_sink == NULL )
+            return 0;
 
-    new_sink->sesp    = ss;
-    new_sink->pdutype = pdutype;
-    new_sink->version = version;
-    new_sink->next    = sinks;
-    sinks = new_sink;
-#endif
+        new_sink->sesp    = ss;
+        new_sink->pdutype = pdutype;
+        new_sink->version = version;
+        new_sink->next    = sinks;
+        sinks = new_sink;
+    }
     return 1;
 }
 
@@ -325,7 +239,7 @@ void snmpd_free_trapsinks (void)
 	 *
 	 *******************/
 
-void convert_v2_to_v1( struct variable_list *vars, struct snmp_pdu *template )
+void convert_v2_to_v1( struct variable_list *vars, struct snmp_pdu *template_pdu )
 {
     struct variable_list *v, *trap_v=NULL, *ent_v=NULL;
     oid  trap_prefix[] = { SNMPV2_TRAPS_PREFIX };
@@ -350,13 +264,13 @@ void convert_v2_to_v1( struct variable_list *vars, struct snmp_pdu *template )
 		 */
     if ( snmp_oid_compare( trap_v->val.objid, OID_LENGTH(trap_prefix), 
     			   trap_prefix,       OID_LENGTH(trap_prefix)) == 0 ) {
-	template->trap_type     = trap_v->val.objid[OID_LENGTH(trap_prefix)]-1;
-	template->specific_type = 0;
+	template_pdu->trap_type     = trap_v->val.objid[OID_LENGTH(trap_prefix)]-1;
+	template_pdu->specific_type = 0;
     }
     else {
 	len = trap_v->val_len /sizeof( oid );
-	template->trap_type     = 6;	/* enterprise specific */
-	template->specific_type = trap_v->val.objid[len-1];
+	template_pdu->trap_type     = 6;	/* enterprise specific */
+	template_pdu->specific_type = trap_v->val.objid[len-1];
     }
 
 		/*
@@ -420,13 +334,17 @@ void send_enterprise_trap_vars (int trap,
 		 *  Create a template PDU, ready for sending
 		 */
     template_pdu = snmp_pdu_create( SNMP_MSG_TRAP );
-    if ( template_pdu == NULL )
+    if ( template_pdu == NULL ) {
+		/* Free memory if value stored dynamically */
+	snmp_set_var_value( &enterprise_var, NULL, 0);
 	return;
+    }
     template_pdu->trap_type     = trap;
     template_pdu->specific_type = specific;
     if ( snmp_clone_mem((void **)&template_pdu->enterprise,
 				enterprise, enterprise_length*sizeof(oid))) {
 	snmp_free_pdu( template_pdu );
+	snmp_set_var_value( &enterprise_var, NULL, 0);
 	return;
     }
     template_pdu->enterprise_length = enterprise_length;
@@ -487,6 +405,7 @@ void send_enterprise_trap_vars (int trap,
 	case SNMP_TRAP_AUTHFAIL:
 		if (snmp_enableauthentraps == SNMP_AUTHENTICATED_TRAPS_DISABLED) {
 		    snmp_free_pdu(template_pdu);
+		    snmp_set_var_value( &enterprise_var, NULL, 0);
 		    return;
 		}
 		snmp_set_var_value( &snmptrap_var,
@@ -556,6 +475,9 @@ void send_enterprise_trap_vars (int trap,
     snmp_call_callbacks(SNMP_CALLBACK_APPLICATION, SNMPD_CALLBACK_SEND_TRAP1,
                         template_pdu);
 
+		/* Free memory if values stored dynamically */
+    snmp_set_var_value( &enterprise_var, NULL, 0);
+    snmp_set_var_value( &snmptrap_var, NULL, 0);
 	/* Ensure we don't free anything we shouldn't */
     if ( last_var )
 	last_var->next_variable = NULL;
@@ -789,7 +711,7 @@ void
 snmpd_parse_config_trapcommunity(const char *word, char *cptr)
 {
     if (snmp_trapcommunity) free(snmp_trapcommunity);
-    snmp_trapcommunity = malloc (strlen(cptr)+1);
+    snmp_trapcommunity = (char *)malloc (strlen(cptr)+1);
     copy_word(cptr, snmp_trapcommunity);
 }
 
