@@ -35,6 +35,10 @@
 #include <arpa/inet.h>
 #endif
 
+#if HAVE_DMALLOC_H
+#include <dmalloc.h>
+#endif
+
 #if HAVE_WINSOCK_H
 #include <winsock.h>
 #endif
@@ -410,7 +414,7 @@ agentx_build_header(struct snmp_pdu *pdu, u_char *bufp, size_t *out_length)
 
     *bufp = 1;			bufp++;		/* version */
     *bufp = pdu->command;	bufp++;		/* type    */
-    *bufp = pdu->flags & AGENTX_MSG_FLAGS_MASK;
+    *bufp = (u_char)(pdu->flags & AGENTX_MSG_FLAGS_MASK);
     				bufp++;		/* AgentX flags */
     *bufp = 0;			bufp++;		/* <reserved> */
     *out_length -=4;
@@ -500,7 +504,7 @@ _agentx_build(struct snmp_session        *session,
 			/* Timeout */
 	    if ( *out_length < 4 )
 	        return -1;
-	    *bufp = pdu->time;		bufp++;
+	    *bufp = (u_char)pdu->time;		bufp++;
 	    *bufp = 0;			bufp++;
 	    *bufp = 0;			bufp++;
 	    *bufp = 0;			bufp++;
@@ -530,7 +534,7 @@ _agentx_build(struct snmp_session        *session,
 			/* Reason */
 	    if ( *out_length < 4 )
 	        return -1;
-	    *bufp = pdu->errstat;	bufp++;
+	    *bufp = (u_char)pdu->errstat;	bufp++;
 	    *bufp = 0;			bufp++;
 	    *bufp = 0;			bufp++;
 	    *bufp = 0;			bufp++;
@@ -545,7 +549,7 @@ _agentx_build(struct snmp_session        *session,
 	case AGENTX_MSG_UNREGISTER:
 	    if ( *out_length < 4 )
 	        return -1;
-	    *bufp = pdu->time;		bufp++;	    /* Timeout (Register only) */
+	    *bufp = (u_char)pdu->time;		bufp++;	    /* Timeout (Register only) */
 	    *bufp = pdu->priority;		bufp++;
 	    range_ptr = bufp;	 	/* Points to the 'range_subid' field */
 	    *bufp = pdu->range_subid;	bufp++;
@@ -865,7 +869,7 @@ agentx_parse_oid( u_char *data, size_t *length, int *inc,
      }
 
 
-     for ( i = 0  ; i<n_subid ; i++ ) {
+     for ( i = 0  ; i < (int)n_subid ; i++ ) {
         oid_ptr[i] = agentx_parse_int( buf_ptr, network_byte_order );
 	buf_ptr += 4;
 	*length -= 4;
@@ -933,14 +937,16 @@ agentx_parse_opaque(u_char *data, size_t *length, int *type,
 	char	c[sizeof(double)];
     } fu;
     int tmp;
-    u_char buf[BUFSIZ];
-    char *cp;
+    u_char *buf;
+    u_char *cp;
 
     cp = agentx_parse_string( data, length,
 				opaque_buf, opaque_len, network_byte_order);
     if ( cp == NULL )
 	return NULL;
 
+	buf = opaque_buf;
+
 #ifdef OPAQUE_SPECIAL_TYPES
     if (( buf[0] != ASN_OPAQUE_TAG1 ) || ( *opaque_len <= 3 ))
 	return cp;	/* Unrecognised opaque type */
@@ -1420,7 +1426,7 @@ agentx_parse(struct snmp_session *session, struct snmp_pdu *pdu, u_char *data, s
 			    return SNMPERR_ASN_PARSE_ERR;
                     }
 		    snmp_pdu_add_variable( pdu, oid_buffer, oid_buf_len,
-				type, buffer, buf_len);
+				(u_char)type, buffer, buf_len);
 
 		    oid_buf_len = MAX_OID_LEN;
 		    buf_len     = BUFSIZ;
