@@ -372,7 +372,9 @@ var_hrstore(struct variable *vp,
     struct pool mbpool, mclpool;
     int i;
 #endif
+#ifdef MBSTAT_SYMBOL
     struct mbstat  mbstat;
+#endif
 #endif	/* solaris2 */
 #endif	/* linux */
     static char string[100];
@@ -520,8 +522,10 @@ var_hrstore(struct variable *vp,
 			long_return = 0;
 			for (i = 0; i < sizeof(mbstat.m_mtypes)/sizeof(mbstat.m_mtypes[0]); i++)
 			    long_return += mbstat.m_mtypes[i];
-#else
+#elif defined(MBSTAT_SYMBOL)
 			long_return = mbstat.m_mbufs;
+#else
+			return NULL;
 #endif
 			break;
 #else	/* linux */
@@ -557,8 +561,10 @@ var_hrstore(struct variable *vp,
 #if HAVE_SYS_POOL_H
 			long_return = (mbpool.pr_nget - mbpool.pr_nput)*mbpool.pr_size
 				+ (mclpool.pr_nget - mclpool.pr_nput)*mclpool.pr_size;
-#else
+#elif defined(MBSTAT_SYMBOL)
 			long_return = mbstat.m_clusters - mbstat.m_clfree;	/* unlikely, but... */
+#else
+			return NULL;
 #endif
 			break;
 #else	/* linux */
@@ -588,7 +594,7 @@ var_hrstore(struct variable *vp,
 #endif
 			long_return = 0;
 			break;
-#if !defined(linux) && !defined(solaris2)
+#if !defined(linux) && !defined(solaris2) && defined(MBSTAT_SYMBOL)
 		case HRS_TYPE_MBUF:
 			long_return = mbstat.m_drops;
 			break;
