@@ -816,6 +816,7 @@ var_ipRouteEntry(struct variable *vp,
     return (u_char *)&long_return;
   case IPROUTEPROTO:
     long_return = Lowentry.ipRouteProto;
+    if (long_return == -1) long_return = 1;
     return (u_char *)&long_return;
   case IPROUTEAGE:
     long_return = Lowentry.ipRouteAge;
@@ -832,7 +833,7 @@ var_ipRouteEntry(struct variable *vp,
 #endif /* solaris2 - var_IProute */
 
 #ifndef solaris2
-static int qsort_compare (RTENTRY **, RTENTRY **);
+static int qsort_compare (void *, void *);
 #endif
 
 #if defined(RTENTRY_4_4) || defined(RTENTRY_RT_NEXT)
@@ -1026,13 +1027,7 @@ static void Route_Scan_Reload (void)
   /*
    *  Sort it!
    */
-  qsort((char *) rthead, rtsize, sizeof(rthead[0]),
-#ifdef __STDC__
-      (int (*)(const void *, const void *))qsort_compare
-#else
-        qsort_compare
-#endif
-    );
+  qsort((char *) rthead, rtsize, sizeof(rthead[0]), qsort_compare);
 }
 
 #else
@@ -1131,14 +1126,7 @@ static void Route_Scan_Reload (void)
 	/*
 	 *  Sort it!
 	 */
-	qsort((char *)rthead,rtsize,sizeof(rthead[0]),
-
-#ifdef __STDC__
-              (int (*)(const void *, const void *)) qsort_compare
-#else
-              qsort_compare
-#endif
-          );
+	qsort((char *)rthead,rtsize,sizeof(rthead[0]), qsort_compare);
 }
 #else
 #ifdef linux
@@ -1244,13 +1232,7 @@ static void Route_Scan_Reload (void)
 	/*
 	 *  Sort it!
 	 */
-	qsort((char *)rthead,rtsize,sizeof(rthead[0]),
-#ifdef __STDC__
-              (int (*)(const void *, const void *)) qsort_compare
-#else
-              qsort_compare
-#endif
-          );
+	qsort((char *)rthead,rtsize,sizeof(rthead[0]), qsort_compare);
 }
 #endif
 #endif
@@ -1261,9 +1243,11 @@ static void Route_Scan_Reload (void)
 /*
  *	Create a host table
  */
-static int qsort_compare(RTENTRY **r1, 
-			 RTENTRY **r2)
+static int qsort_compare(void *v1,
+			 void *v2)
 {
+	RTENTRY **r1 = (RTENTRY **) v1;
+	RTENTRY **r2 = (RTENTRY **) v2;
 #if NEED_KLGETSA
 	register u_long dst1 = ntohl(klgetsa((struct sockaddr_in *)(*r1)->rt_dst)->sin_addr.s_addr);
 	register u_long dst2 = ntohl(klgetsa((struct sockaddr_in *)(*r2)->rt_dst)->sin_addr.s_addr);
