@@ -179,6 +179,7 @@ smux_free_peer_auth(void)
 		free(Auths[i]);
 		Auths[i] = NULL;
 	}
+	nauths = 0;
 }
 
 void
@@ -266,8 +267,7 @@ var_smux(struct variable *vp,
 	u_char *valptr, val_type;
 	smux_reg *rptr;
 
-	*write_method = NULL;
-
+	*write_method = var_smux_write; 
 	/* search the active registration list */
 	for (rptr = ActiveRegs; rptr; rptr = rptr->sr_next) {
 		if (!compare_tree(vp->name, vp->namelen, rptr->sr_name,
@@ -279,7 +279,6 @@ var_smux(struct variable *vp,
 	else if (exact && (*length < rptr->sr_name_len))
 		return NULL;
 
-	*write_method = var_smux_write; 
 	valptr = smux_snmp_process(exact, name, length,
 	    var_len, &val_type, rptr->sr_fd);
 
