@@ -66,6 +66,8 @@ void init_vmstat_netbsd1(void)
     {CPURAWNICE, ASN_COUNTER, RONLY, var_extensible_vmstat, 1, {CPURAWNICE}},
     {CPURAWSYSTEM, ASN_COUNTER, RONLY, var_extensible_vmstat, 1, {CPURAWSYSTEM}},
     {CPURAWIDLE, ASN_COUNTER, RONLY, var_extensible_vmstat, 1, {CPURAWIDLE}},
+    {CPURAWKERNEL, ASN_COUNTER, RONLY, var_extensible_vmstat, 1, {CPURAWKERNEL}},
+    {CPURAWINTR, ASN_COUNTER, RONLY, var_extensible_vmstat, 1, {CPURAWINTR}},
 
 /* Future use: */
 /*
@@ -240,6 +242,12 @@ unsigned char *var_extensible_vmstat(struct variable *vp,
     case CPURAWIDLE:
 	long_ret = cpu_new[CP_IDLE];
 	return((u_char *) (&long_ret));
+    case CPURAWKERNEL:
+	long_ret = cpu_new[CP_SYS];
+	return((u_char *) (&long_ret));
+    case CPURAWINTR:
+	long_ret = cpu_new[CP_INTR];
+	return((u_char *) (&long_ret));
 /* reserved for future use */
 /*
   case ERRORFLAG:
