@@ -15,9 +15,11 @@ struct agent_snmp_session {
     struct variable_list *start, *end;
     struct snmp_session  *session;
     struct snmp_pdu      *pdu;
+    struct snmp_pdu      *orig_pdu;
     int		rw;
     int		exact;
     int		status;
+    int		index;
     
     struct request_list *outstanding_requests;
     struct agent_snmp_session *next;
@@ -26,9 +28,11 @@ struct agent_snmp_session {
 /* config file parsing routines */
 int handle_snmp_packet(int, struct snmp_session *, int, struct snmp_pdu *, void *);
 int handle_next_pass( struct agent_snmp_session *);
-int  handle_var_list( struct agent_snmp_session *);
+int handle_var_list( struct agent_snmp_session *);
+int handle_one_var( struct agent_snmp_session *, struct variable_list *varbind_ptr);
 void snmp_agent_parse_config (char *, char *);
 struct agent_snmp_session  *init_agent_snmp_session( struct snmp_session *, struct snmp_pdu *);
+void free_agent_snmp_session( struct agent_snmp_session * );
 int getNextSessID(void);
 int init_master_agent(int dest_port,
                        int (*pre_parse) (struct snmp_session *, snmp_ipaddr),
