@@ -46,7 +46,20 @@ SOFTWARE.
 #endif
 
 #include <sys/types.h>
-
+#if TIME_WITH_SYS_TIME
+# ifdef WIN32
+#  include <sys/timeb.h>
+# else
+#  include <sys/time.h>
+# endif
+# include <time.h>
+#else
+# if HAVE_SYS_TIME_H
+#  include <sys/time.h>
+# else
+#  include <time.h>
+# endif
+#endif
 #if HAVE_SYS_SELECT_H
 #include <sys/select.h>
 #endif
