@@ -84,6 +84,9 @@ SOFTWARE.
 #if HAVE_FCNTL_H
 #include <fcntl.h>
 #endif
+#if HAVE_PROCESS_H  /* Win32-getpid */
+#include <process.h>
+#endif
 #include <signal.h>
 #include <errno.h>
 
@@ -108,7 +111,6 @@ SOFTWARE.
 #include "lcd_time.h"
 #include "transform_oids.h"
 #include "snmpv3.h"
-#include "snmp_alarm.h"
 #include "default_store.h"
 
 #define DS_APP_NUMERIC_IP  1
@@ -457,7 +459,7 @@ int snmp_input(int op,
 	            (void) format_plain_trap (out_bfr, SPRINT_MAX_LEN, pdu);
 	        else
 		    (void) format_trap (out_bfr, SPRINT_MAX_LEN, trap1_fmt_str, pdu);
-                snmp_log(LOG_INFO, out_bfr);
+                snmp_log(LOG_INFO, "%s", out_bfr);
 	    }
 	    if (Syslog && (pdu->trap_type != SNMP_TRAP_AUTHFAIL || dropauth == 0)) {
 	    	varbufidx=0;
@@ -516,7 +518,7 @@ int snmp_input(int op,
                 else
                     (void) format_trap (out_bfr, SPRINT_MAX_LEN,
                                         trap2_fmt_str, pdu);
-                snmp_log(LOG_INFO, out_bfr);
+                snmp_log(LOG_INFO, "%s", out_bfr);
 	    }
 	    if (Syslog) {
 	    	varbufidx=0;
@@ -659,6 +661,7 @@ int main(int argc, char *argv[])
     int dofork=1;
     char *cp;
     int tcp=0;
+    char *trap1_fmt_str_remember = NULL;
 #if HAVE_GETPID
 	FILE           *PID;
         char *pid_file = NULL;
@@ -821,7 +824,7 @@ int main(int argc, char *argv[])
             break;
 
 	case 'F':
-	    trap1_fmt_str = optarg;
+	    trap1_fmt_str_remember = optarg;
 	    break;
 
 	default:
@@ -841,6 +844,10 @@ int main(int argc, char *argv[])
 
     /* Initialize the world. Create initial user */
     init_snmp("snmptrapd");
+    if (trap1_fmt_str_remember) {
+        free_trap1_fmt();
+        trap1_fmt_str = strdup(trap1_fmt_str_remember);
+    }
 
 #ifndef WIN32
     /* fork the process to the background if we are not printing to stdout */
@@ -950,6 +957,10 @@ int main(int argc, char *argv[])
 	    if (Syslog)
 		syslog(LOG_INFO, "Snmptrapd reconfiguring");
 	    update_config();
+            if (trap1_fmt_str_remember) {
+                free_trap1_fmt();
+                trap1_fmt_str = strdup(trap1_fmt_str_remember);
+            }
 	    reconfig = 0;
 	}
 	numfds = 0;
@@ -970,7 +981,10 @@ int main(int argc, char *argv[])
 		snmp_timeout();
 		break;
 	    case -1:
+		if (errno == EINTR)
+			continue;
 	        snmp_log_perror("select");
+		running = 0;
 		break;
 	    default:
 		fprintf(stderr, "select returned %d\n", count);
