@@ -213,7 +213,6 @@ sc_generate_keyed_hash(	oid	*authtype,	size_t authtypelen,
   u_char	 buf[SNMP_MAXBUF_SMALL];
 #if  defined(USE_OPENSSL)
   int		 buf_len = sizeof(buf);
-  u_char	*bufp = buf;
 #endif
   
   DEBUGTRACE;
@@ -495,8 +494,7 @@ sc_encrypt(	oid    *privtype,	size_t privtypelen,
 #if defined(USE_OPENSSL) 
 {
 	int		rval	= SNMPERR_SUCCESS;
-	u_int		transform,
-			properlength,
+	u_int		properlength,
 			properlength_iv;
 	u_char		pad_block[32];  /* bigger than anything I need */
 	u_char          my_iv[32];      /* ditto */
@@ -519,7 +517,7 @@ sc_encrypt(	oid    *privtype,	size_t privtypelen,
 	{
 		QUITFUN(SNMPERR_GENERR, sc_encrypt_quit);
 	}
-	else if ( ptlen >= *ctlen) { 
+	else if ( ptlen > *ctlen) { 
 		QUITFUN(SNMPERR_GENERR, sc_encrypt_quit);
 	}
 
@@ -554,20 +552,17 @@ sc_encrypt(	oid    *privtype,	size_t privtypelen,
 		QUITFUN(SNMPERR_GENERR, sc_encrypt_quit);
 	}
 
-	else if ( (keylen<properlength) || (ivlen<properlength_iv) ) {
-		QUITFUN(SNMPERR_GENERR, sc_encrypt_quit);
-	}
-
 /* now calculate the padding needed */
 	pad = pad_size - (ptlen % pad_size);
+	plast = (int) ptlen - (pad_size - pad);
+        if (pad == pad_size) pad = 0;
 	if (ptlen + pad > *ctlen) { 
 		QUITFUN(SNMPERR_GENERR, sc_encrypt_quit); /* not enough space */
 	}
-	memset(pad_block, 0, sizeof(pad_block));
-	plast = (int) ptlen - (pad_size - pad);
-	if (pad > 0)  /* copy data into pad block if needed */
+	if (pad > 0) { /* copy data into pad block if needed */
 		memcpy( pad_block, plaintext + plast, pad_size - pad);
-	memset(&pad_block[pad_size-pad], pad, pad); /* filling in padblock */
+                memset(&pad_block[pad_size-pad], pad, pad); /* filling in padblock */
+        }
 
 	memset(my_iv, 0, sizeof(my_iv));
 
@@ -578,11 +573,15 @@ sc_encrypt(	oid    *privtype,	size_t privtypelen,
 		memcpy(my_iv, iv, ivlen);
 		/* encrypt the data */
 		des_ncbc_encrypt(plaintext, ciphertext, plast, key_sch, 
-				 (des_cblock *) &my_iv, DES_ENCRYPT);
-		/* then encrypt the pad block */
-		des_ncbc_encrypt(pad_block, ciphertext+plast, pad_size, 
-				 key_sch, (des_cblock *)&my_iv, DES_ENCRYPT);
-		*ctlen = plast + pad_size;
+				 (des_cblock *) my_iv, DES_ENCRYPT);
+                if (pad > 0) {
+                    /* then encrypt the pad block */
+                    des_ncbc_encrypt(pad_block, ciphertext+plast, pad_size, 
+                                     key_sch, (des_cblock *)my_iv, DES_ENCRYPT);
+                    *ctlen = plast + pad_size;
+                } else {
+                    *ctlen = plast;
+                }
 	}
 sc_encrypt_quit:
 	/* clear memory just in case */
@@ -646,7 +645,6 @@ sc_decrypt(	oid    *privtype,	size_t privtypelen,
 {
 
 	int rval = SNMPERR_SUCCESS;
-	int i, j; 
 	u_char *my_iv[32];
 	des_key_schedule key_sch;
 	des_cblock  key_struct;
@@ -696,7 +694,8 @@ sc_decrypt(	oid    *privtype,	size_t privtypelen,
 	
 		memcpy(my_iv, iv, ivlen);
 		des_cbc_encrypt(ciphertext, plaintext, ctlen, key_sch, 
-				(des_cblock *) &my_iv, DES_DECRYPT);
+				(des_cblock *) my_iv, DES_DECRYPT);
+                *ptlen = ctlen;
 	}
 
 /* exit cond */
